/*
 * IRC - Internet Relay Chat, ports/nt/os/os.c
 * Port library file for NT version of ircd.
 * 1997 Thomas Helvey <tomh@inxpress.net>
 *
 * See file AUTHORS in IRC package for additional names of
 * the programmers.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <winsock2.h>
#include <string.h>
#include <errno.h>

/*
 * All Windows Sockets error constants are biased by WSABASEERR from
 * the "normal"
 * #define WSABASEERR              10000
 */
#define UNKNOWN_ERROR "Unknown error"

static const char* ws_errlist[] = {
  /*
   * Windows Sockets definitions of regular Microsoft C error constants
   */
  "No Error",
  UNKNOWN_ERROR,              /* (WSABASEERR+1) */
  UNKNOWN_ERROR,              /* (WSABASEERR+2) */
  UNKNOWN_ERROR,              /* (WSABASEERR+3) */
  "Interrupted system call",  /* WSAEINTR       (WSABASEERR+4) */
  UNKNOWN_ERROR,              /* (WSABASEERR+5) */
  UNKNOWN_ERROR,              /* (WSABASEERR+6) */
  UNKNOWN_ERROR,              /* (WSABASEERR+7) */
  UNKNOWN_ERROR,              /* (WSABASEERR+8) */
  "Bad file descriptor",      /* WSAEBADF       (WSABASEERR+9) */
  UNKNOWN_ERROR,              /* (WSABASEERR+10) */
  UNKNOWN_ERROR,              /* (WSABASEERR+11) */
  UNKNOWN_ERROR,              /* (WSABASEERR+12) */
  "Permission denied",        /* WSAEACCES      (WSABASEERR+13) */
  "Bad address",              /* WSAEFAULT      (WSABASEERR+14) */
  UNKNOWN_ERROR,              /* (WSABASEERR+15) */
  UNKNOWN_ERROR,              /* (WSABASEERR+16) */
  UNKNOWN_ERROR,              /* (WSABASEERR+17) */
  UNKNOWN_ERROR,              /* (WSABASEERR+18) */
  UNKNOWN_ERROR,              /* (WSABASEERR+19) */
  UNKNOWN_ERROR,              /* (WSABASEERR+20) */
  UNKNOWN_ERROR,              /* (WSABASEERR+21) */
  "Invalid argument",         /* WSAEINVAL      (WSABASEERR+22) */
  UNKNOWN_ERROR,              /* (WSABASEERR+23) */
  "Too many open files",      /* WSAEMFILE      (WSABASEERR+24) */
  UNKNOWN_ERROR,              /* (WSABASEERR+25) */
  UNKNOWN_ERROR,              /* (WSABASEERR+26) */
  UNKNOWN_ERROR,              /* (WSABASEERR+27) */
  UNKNOWN_ERROR,              /* (WSABASEERR+28) */
  UNKNOWN_ERROR,              /* (WSABASEERR+29) */
  UNKNOWN_ERROR,              /* (WSABASEERR+30) */
  UNKNOWN_ERROR,              /* (WSABASEERR+31) */
  UNKNOWN_ERROR,              /* (WSABASEERR+32) */
  UNKNOWN_ERROR,              /* (WSABASEERR+33) */
  UNKNOWN_ERROR,              /* (WSABASEERR+34) */

  /*
   * Windows Sockets definitions of regular Berkeley error constants
   */
  "Operation would block",              /* WSAEWOULDBLOCK       (WSABASEERR+35) */
  "Operation now in progress",          /* WSAEINPROGRESS       (WSABASEERR+36) */
  "Operation already in progress",      /* WSAEALREADY          (WSABASEERR+37) */
  "Socket operation on nonsocket",      /* WSAENOTSOCK          (WSABASEERR+38) */
  "Destination address required",       /* WSAEDESTADDRREQ      (WSABASEERR+39) */
  "Message too long",                   /* WSAEMSGSIZE          (WSABASEERR+40) */
  "Protocol wrong type for socket",     /* WSAEPROTOTYPE        (WSABASEERR+41) */
  "Bad protocol option",                /* WSAENOPROTOOPT       (WSABASEERR+42) */
  "Protocol not supported",             /* WSAEPROTONOSUPPORT   (WSABASEERR+43) */
  "Socket type not supported",          /* WSAESOCKTNOSUPPORT   (WSABASEERR+44) */
  "Operation not supported on socket",  /* WSAEOPNOTSUPP        (WSABASEERR+45) */
  "Protocol family not supported",      /* WSAEPFNOSUPPORT      (WSABASEERR+46) */
  "Address family not supported by protocol family", /* WSAEAFNOSUPPORT (WSABASEERR+47) */
  "Address already in use",             /* WSAEADDRINUSE        (WSABASEERR+48) */
  "Cannot assign requested address",    /* WSAEADDRNOTAVAIL     (WSABASEERR+49) */
  "Network is down",                    /* WSAENETDOWN          (WSABASEERR+50) */
  "Network is unreachable",             /* WSAENETUNREACH       (WSABASEERR+51) */
  "Net dropped connection or reset",    /* WSAENETRESET         (WSABASEERR+52) */
  "Software caused connection abort",   /* WSAECONNABORTED      (WSABASEERR+53) */
  "Connection reset by peer",           /* WSAECONNRESET        (WSABASEERR+54) */
  "No buffer space available",          /* WSAENOBUFS           (WSABASEERR+55) */
  "Socket is already connected",        /* WSAEISCONN           (WSABASEERR+56) */
  "Socket is not connected",            /* WSAENOTCONN          (WSABASEERR+57) */
  "Cannot send after socket shutdown",  /* WSAESHUTDOWN         (WSABASEERR+58) */
  "Too many references, cannot splice", /* WSAETOOMANYREFS      (WSABASEERR+59) */
  "Connection timed out",               /* WSAETIMEDOUT         (WSABASEERR+60) */
  "Connection refused",                 /* WSAECONNREFUSED      (WSABASEERR+61) */
  "Too many levels of symbolic links",  /* WSAELOOP             (WSABASEERR+62) */
  "File name too long",                 /* WSAENAMETOOLONG      (WSABASEERR+63) */
  "Host is down",                       /* WSAEHOSTDOWN         (WSABASEERR+64) */
  "No route to host",                   /* WSAEHOSTUNREACH      (WSABASEERR+65) */
  "Directory not empty",                /* WSAENOTEMPTY         (WSABASEERR+66) */
  "Too many processes",                 /* WSAEPROCLIM          (WSABASEERR+67) */
  "Too many users",                     /* WSAEUSERS            (WSABASEERR+68) */
  "Disk quota exceeded",                /* WSAEDQUOT            (WSABASEERR+69) */
  "Stale NFS file handle",              /* WSAESTALE            (WSABASEERR+70) */
  "Too many levels of remote in path",  /* WSAEREMOTE           (WSABASEERR+71) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+72) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+73) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+74) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+75) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+76) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+77) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+78) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+79) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+80) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+81) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+82) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+83) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+84) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+85) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+86) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+87) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+88) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+89) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+90) */

  /*
   * Extended Windows Sockets error constant definitions
   */
  "Network subsystem not available",    /* WSASYSNOTREADY          (WSABASEERR+91) */
  "WINSOCK DLL version out of range",   /* WSAVERNOTSUPPORTED      (WSABASEERR+92) */
  "Successful WSAStartup not yet performed", /* WSANOTINITIALISED  (WSABASEERR+93) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+94) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+95) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+96) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+97) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+98) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+99) */
  UNKNOWN_ERROR,                      /* (WSABASEERR+100) */
  "Disconnected",                       /* WSAEDISCON              (WSABASEERR+101) */
  "No more data",                       /* WSAENOMORE              (WSABASEERR+102) */
  "Blocking call canceled",             /* WSAECANCELLED           (WSABASEERR+103) */
  "Invalid process table",              /* WSAEINVALIDPROCTABLE    (WSABASEERR+104) */
  "Invalid service provider",           /* WSAEINVALIDPROVIDER     (WSABASEERR+105) */
  "Service provider init failed",       /* WSAEPROVIDERFAILEDINIT  (WSABASEERR+106) */
  "System call failure",                /* WSASYSCALLFAILURE       (WSABASEERR+107) */
  "Service not found",                  /* WSASERVICE_NOT_FOUND    (WSABASEERR+108) */
  "Service type not found",             /* WSATYPE_NOT_FOUND       (WSABASEERR+109) */
  "No more data",                       /* WSA_E_NO_MORE           (WSABASEERR+110) */
  "Blocking call canceled",             /* WSA_E_CANCELLED         (WSABASEERR+111) */
  "Connection refused",                 /* WSAEREFUSED             (WSABASEERR+112) */
  0,
};

/*
 * Error return codes from gethostbyname() and gethostbyaddr()
 * (when using the resolver). Note that these errors are
 * retrieved via WSAGetLastError() and must therefore follow
 * the rules for avoiding clashes with error numbers from
 * specific implementations or language run-time systems.
 * For this reason the codes are based at WSABASEERR+1001.
 * Note also that [WSA]NO_ADDRESS is defined only for
 * compatibility purposes.
 */

static const char* wsh_errlist[] = {
  "No error",                               /* (WSABASEERR+1000) */
  "Host not found",                         /* WSAHOST_NOT_FOUND  (WSABASEERR+1001) */
  "Nonauthoritative host not found",        /* WSATRY_AGAIN       (WSABASEERR+1002) */
  "Non-recoverable error",                  /* WSANO_RECOVERY     (WSABASEERR+1003) */
  "Valid name, no data record of requested type", /* WSANO_DATA   (WSABASEERR+1004) */
  "No address, look for MX record",         /* WSANO_ADDRESS       WSANO_DATA */
  0,
};

const char* nt_strerror(int err_no)
{
  if (err_no >= WSABASEERR && err_no <= WSAEREFUSED)
    return ws_errlist[err_no - WSABASEERR];
  else if (err_no >= (WSABASEERR + 1000) && err_no <= WSANO_DATA)
    return wsh_errlist[err_no - (WSABASEERR + 1000)];
  else
    return strerror(err_no);
}
